<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:transform version="1.0"
xmlns:tc="http://www.bom.gov.au/dtd/tc"
xmlns:gml="http://www.opengis.net/gml"
xmlns:exsl="http://exslt.org/common"
extension-element-prefixes="exsl"
xmlns:maximum-pick-bigger="f:maximum-pick-bigger"
xmlns:maximum-own-compare="f:maximum-own-compare"
xmlns:maximum-len-compare="f:maximum-len-compare"
xmlns:maximum-type-compare="f:maximum-type-compare"
exclude-result-prefixes="xsl maximum-own-compare maximum-pick-bigger"
xmlns:fn="http://www.w3.org/2005/02/xpath-functions"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="xml" omit-xml-declaration="yes" indent="yes"/>

<xsl:template match="/featureCollection">
<GMLFeatureClassList>
    <GMLFeatureClass>
        <Name>GML2GFS</Name>
        <ElementPath>GML2GFS</ElementPath>
        <DatasetSpecificInfo>
            <FeatureCount>
                <xsl:value-of select="count(gml:featureMember)"/>
            </FeatureCount>
            <ExtentXMin>
                <xsl:value-of select="gml:boundedBy/gml:Box/gml:coord[1]/gml:X"/>
            </ExtentXMin>
            <ExtentXMax>
                <xsl:value-of select="gml:boundedBy/gml:Box/gml:coord[2]/gml:X"/>
            </ExtentXMax>
            <ExtentYMin>
                <xsl:value-of select="gml:boundedBy/gml:Box/gml:coord[1]/gml:Y"/>
            </ExtentYMin>
            <ExtentYMax>
                <xsl:value-of select="gml:boundedBy/gml:Box/gml:coord[2]/gml:Y"/>
            </ExtentYMax>
        </DatasetSpecificInfo>
        <xsl:for-each select="gml:featureMember[last()]//*[geometryProperty]/*">
          <xsl:if test="name() != 'geometryProperty'">
            <PropertyDefn>
                <xsl:variable name="name" select="name()"/>
                <Name>
                    <xsl:value-of select="$name"/>
                </Name>
                <ElementPath>
                    <xsl:value-of select="$name"/>
                </ElementPath>
                    <xsl:variable name="worstTypeValue">
                        <xsl:call-template name="maximumType">
                            <xsl:with-param name="pList" select="/featureCollection/gml:featureMember//*[name() = $name and name() != 'geometryProperty']"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="worstType">
                        <xsl:choose>
                          <xsl:when test="round(concat(substring-before($worstTypeValue, '.'),substring-after($worstTypeValue, '.')))">Float</xsl:when>
                          <xsl:when test="string(round($worstTypeValue)) = 'NaN'">String</xsl:when>
                          <xsl:otherwise>Integer</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                <Type>
                    <xsl:value-of select="$worstType"/>
                </Type>
                <xsl:if test="$worstType = 'String'">
                  <Width>
                    <xsl:variable name="longestString">
                        <xsl:call-template name="maximumLength">
                            <xsl:with-param name="pList" select="/featureCollection/gml:featureMember//*[name() = $name and name() != 'geometryProperty']"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:value-of select="string-length($longestString)"/>
                  </Width>
                </xsl:if>
            </PropertyDefn>
          </xsl:if>
        </xsl:for-each>
    </GMLFeatureClass>
</GMLFeatureClassList>
</xsl:template>

<xsl:template name="foldl">
<xsl:param name="pFunc" select="/.."/>
  <xsl:param name="pA0"/>
  <xsl:param name="pList" select="/.."/>

  <xsl:choose>
     <xsl:when test="not($pList)">
        <xsl:copy-of select="$pA0"/>
     </xsl:when>
     <xsl:otherwise>
       <xsl:variable name="vFunResult">
          <xsl:apply-templates select="$pFunc[1]">
            <xsl:with-param name="arg0" select="$pFunc[position() > 1]"/>
            <xsl:with-param name="arg1" select="$pA0"/>
            <xsl:with-param name="arg2" select="$pList[1]"/>
          </xsl:apply-templates>
       </xsl:variable>

       <xsl:call-template name="foldl">
          <xsl:with-param name="pFunc" select="$pFunc"/>
          <xsl:with-param name="pList" select="$pList[position() > 1]"/>
          <xsl:with-param name="pA0" select="$vFunResult"/>

       </xsl:call-template>
     </xsl:otherwise>
  </xsl:choose>

</xsl:template> 

   <maximum-pick-bigger:maximum-pick-bigger/>
   <maximum-own-compare:maximum-own-compare/>
   <maximum-len-compare:maximum-len-compare/>
   <maximum-type-compare:maximum-type-compare/>

    <xsl:template name="maximumType">
      <xsl:param name="pList" select="/.."/>
      <xsl:param name="pCMPFun" select="/.."/>

      <xsl:variable name="vdfCMPFun" 
           select="document('')/*/maximum-type-compare:*"/>
      <xsl:variable name="vFoldFun" 
           select="document('')/*/maximum-pick-bigger:*[1]"/>
   
      <xsl:if test="$pList">
         <xsl:variable name="vCMPFun" select="$pCMPFun |
$vdfCMPFun[not($pCMPFun)]"/>
         <xsl:variable name="vFuncList">
            <xsl:copy-of select="$vFoldFun"/> <!-- Pick Bigger -->
            <xsl:copy-of select="$vCMPFun"/>  <!-- Compare -->
         </xsl:variable>

         <xsl:call-template name="foldl">
            <xsl:with-param name="pFunc" select="exsl:node-set($vFuncList)/*"/>
            <xsl:with-param name="pList" select="$pList"/>
            <xsl:with-param name="pA0" select="$pList[1]"/>
         </xsl:call-template>
      </xsl:if>
    </xsl:template>

    <xsl:template name="maximumLength">
      <xsl:param name="pList" select="/.."/>
      <xsl:param name="pCMPFun" select="/.."/>

      <xsl:variable name="vdfCMPFun" 
           select="document('')/*/maximum-len-compare:*"/>
      <xsl:variable name="vFoldFun" 
           select="document('')/*/maximum-pick-bigger:*[1]"/>
   
      <xsl:if test="$pList">
         <xsl:variable name="vCMPFun" select="$pCMPFun |
$vdfCMPFun[not($pCMPFun)]"/>
         <xsl:variable name="vFuncList">
            <xsl:copy-of select="$vFoldFun"/> <!-- Pick Bigger -->
            <xsl:copy-of select="$vCMPFun"/>  <!-- Compare -->
         </xsl:variable>

         <xsl:call-template name="foldl">
            <xsl:with-param name="pFunc" select="exsl:node-set($vFuncList)/*"/>
            <xsl:with-param name="pList" select="$pList"/>
            <xsl:with-param name="pA0" select="$pList[1]"/>
         </xsl:call-template>
      </xsl:if>
    </xsl:template>


    <xsl:template name="maximum">
      <xsl:param name="pList" select="/.."/>
      <xsl:param name="pCMPFun" select="/.."/>

      <xsl:variable name="vdfCMPFun" 
           select="document('')/*/maximum-own-compare:*[name='isGreaterDefault']"/>
      <xsl:variable name="vFoldFun" 
           select="document('')/*/maximum-pick-bigger:*[1]"/>
   
      <xsl:if test="$pList">
         <xsl:variable name="vCMPFun" select="$pCMPFun |
$vdfCMPFun[not($pCMPFun)]"/>
         <xsl:variable name="vFuncList">
            <xsl:copy-of select="$vFoldFun"/> <!-- Pick Bigger -->
            <xsl:copy-of select="$vCMPFun"/>  <!-- Compare -->
         </xsl:variable>

         <xsl:call-template name="foldl">
            <xsl:with-param name="pFunc" select="exsl:node-set($vFuncList)/*"/>
            <xsl:with-param name="pList" select="$pList"/>
            <xsl:with-param name="pA0" select="$pList[1]"/>
         </xsl:call-template>
      </xsl:if>
    </xsl:template>




    <xsl:template name="pickBigger" match="maximum-pick-bigger:*">
         <xsl:param name="arg0"/>
         <xsl:param name="arg1"/>
         <xsl:param name="arg2"/>

         <xsl:variable name="vIsGreater">
           <xsl:apply-templates select="$arg0">
             <xsl:with-param name="arg1" select="$arg1"/>
             <xsl:with-param name="arg2" select="$arg2"/>
           </xsl:apply-templates>
         </xsl:variable>
         <xsl:choose>
           <xsl:when test="$vIsGreater = 1">
             <xsl:copy-of select="$arg1"/>
           </xsl:when>
           <xsl:otherwise>
             <xsl:copy-of select="$arg2"/>
           </xsl:otherwise>
         </xsl:choose>
    </xsl:template>

    <xsl:template name="isGreaterType" match="maximum-type-compare:*">
         <xsl:param name="arg1"/>
         <xsl:param name="arg2"/>

         <xsl:choose>
          <!--  If arg1 is a float, '1' -->
          <xsl:when test="round(concat(substring-before($arg1, '.'),substring-after($arg1, '.')))">1</xsl:when>
          <!--  If arg1 is a string, '0' -->
          <xsl:when test="string(round($arg1)) = 'NaN'">0</xsl:when>
          <!--  If arg2 is a float, '0' -->
          <xsl:when test="round(concat(substring-before($arg2, '.'),substring-after($arg1, '.')))">0</xsl:when>
          <!-- otherwise, 1 -->
          <xsl:otherwise>1</xsl:otherwise>
         </xsl:choose>
    </xsl:template>

    <xsl:template name="isGreaterDefault" match="maximum-own-compare:*">
         <xsl:param name="arg1"/>
         <xsl:param name="arg2"/>

         <xsl:choose>
          <xsl:when test="$arg1 > $arg2">1</xsl:when>
          <xsl:otherwise>0</xsl:otherwise>
         </xsl:choose>
    </xsl:template>


    <xsl:template name="isGreaterLength" match="maximum-len-compare:*">
         <xsl:param name="arg1"/>
         <xsl:param name="arg2"/>

         <xsl:choose>
          <xsl:when test="string-length($arg1) > string-length($arg2)">1</xsl:when>
          <xsl:otherwise>0</xsl:otherwise>
         </xsl:choose>
    </xsl:template>


</xsl:transform>
